#include <windows.h>
#include <accCtrl.h>
#include <aclapi.h>
#include <stdio.h>

int wmain(int argc, WCHAR **argv)
{
	// Directory object to be assigned new DACL
	LPTSTR pszPath = L"\\\\?\\C:\\AnotherDACLDir";
	EXPLICIT_ACCESS ea = {0};
	PACL acl = NULL;
	SECURITY_ATTRIBUTES sa = {0};
	SID_IDENTIFIER_AUTHORITY SIDAuthWorld = SECURITY_WORLD_SID_AUTHORITY;
	// Everyone SID
	PSID everyone_sid = NULL;
	DWORD dwRetVal = 0;
	PSECURITY_DESCRIPTOR sd = {0};

	// Verify the object validity
	if(pszPath == NULL)
	{
		wprintf(L"The object name is not valid!\n");
		return ERROR_INVALID_PARAMETER;
	}
	else
		wprintf(L"The object name is valid!\n");

	// allocates and initializes a security identifier (SID),
	// and can be up to eight subauthorities
	// This function creates a SID with a 32-bit RID value.
	// For applications that require longer RID values, use CreateWellKnownSid()
	// - http://msdn.microsoft.com/en-us/library/aa379650%28VS.85%29.aspx
	if(AllocateAndInitializeSid(&SIDAuthWorld,
		1,
		SECURITY_WORLD_RID,
		0, 0, 0, 0, 0, 0, 0,
		&everyone_sid) != 0)

		wprintf(L"SID for Everyone was allocated and initialized!\n");
	else
	{
		wprintf(L"AllocateAndInitializeSid() failed, error %u\n", GetLastError());
		exit(1);
	}
	
	// Initializes SECURITY_ATTRIBUTES
	SecureZeroMemory(&ea, sizeof(EXPLICIT_ACCESS));

	ea.grfAccessPermissions = SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL;
	ea.grfAccessMode = SET_ACCESS;
	ea.grfInheritance = NO_INHERITANCE;
	ea.Trustee.TrusteeForm = TRUSTEE_IS_SID;
	ea.Trustee.TrusteeType = TRUSTEE_IS_WELL_KNOWN_GROUP;
	ea.Trustee.ptstrName  = (LPWSTR)everyone_sid;
	
	dwRetVal = SetEntriesInAcl(1, &ea, NULL, &acl);

	if(dwRetVal == ERROR_SUCCESS)
		wprintf(L"Entries in the ACL has been set successfully!\n");
	else
	{
		wprintf(L"Faile to set the ACL entries, error %u\n", dwRetVal);
		exit(1);
	}
	
	sd = (PSECURITY_DESCRIPTOR)LocalAlloc(LPTR,SECURITY_DESCRIPTOR_MIN_LENGTH);

	// initializes a new security descriptor
	if(InitializeSecurityDescriptor(sd, SECURITY_DESCRIPTOR_REVISION) != 0)
		wprintf(L"Security descriptor was initialized!\n");
	else
	{
		wprintf(L"Failed to Security descriptor, error %u\n", GetLastError());
		exit(1);
	}

	// sets information in a discretionary access control list (DACL)
	// If a DACL is already present in the security descriptor, the DACL is replaced.
	if(SetSecurityDescriptorDacl(sd, TRUE, acl, FALSE) != 0)
		wprintf(L"The security descriptor was set successfully!\n");
	else
	{
		wprintf(L"Failed to set security descriptor, error %u\n", GetLastError());
		exit(1);
	}
	
	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = sd;
	sa.bInheritHandle = FALSE;

	// Create directory object with new DACL
	if(CreateDirectory(pszPath, &sa) != 0)
		wprintf(L"\"%s\" was successfully created with new DACL!\n", pszPath);
	else
	{
		wprintf(L"Failed to create \"%s\" with new DACL, error %u\n", pszPath, GetLastError());
		exit(1);
	}

	// Relesing the allocated resources
	FreeSid(everyone_sid);
	LocalFree(sd);
	LocalFree(acl);

	return 0;
}